@@ -1,18 +1,17 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) [2008] [Carl Chenet]
-# Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le
-# modifier au titre des clauses de la Licence Publique Générale GNU, telle que
-# publiée par la Free Software Foundation ; soit la version 2 de la Licence,
-# ou (à votre discrétion) une version ultérieure quelconque.
+# Copyright © 2008 Carl Chenet <chaica@ohmytux.com>
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# any later version.
 #
-# Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE
-# GARANTIE ; sans même une garantie implicite de COMMERCIABILITE ou DE
-# CONFORMITE A UNE UTILISATION PARTICULIERE.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
-# Voir la Licence Publique Générale GNU pour plus de détails. Vous devriez
-# avoir reçu un exemplaire de la Licence Publique Générale GNU avec ce
-# programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation
-# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #Ouvre un terminal sur une machine distante
 """Ouvre un terminal sur une machine distante"""
@@ -21,6 +20,8 @@ import sys
 import stat
 from os import linesep, chmod, sep
 from os.path import expanduser, abspath, join
+ 
+from optionstunnel import OptionsTunnel
 
 SSHOPTS = '-o NoHostAuthenticationForLocalhost=yes -o StrictHostKeyChecking=no'
 
@@ -29,8 +30,13 @@ class Terminal:
 
     def __init__(self, opts):
         self._ordres = []
-        self._premiereligne = True
-        self._commande = False
+        self._est_premiereligne = True
+        self._a_commande = False
+        self._a_tunnel = False
+        self._port = ''
+        self._pdest = ''
+        self._a_optstunnel = False
+        self._dernierssh = 0
         self._destination = ''
         self._machinefinale = ''
         self._extfichier = '.sh'
@@ -87,7 +93,7 @@ exceeds 255 characters')
                             print _("Your hostname size \
 exceeds 64 characters")
                             sys.exit(1)
-                    if self._ordres[num].split()[-1] == '-c'+linesep or \
+                    if self._ordres[num].split()[-1] == '-c'+ linesep or \
                         self._ordres[num].split()[-1] == '-c':
                         break 
 
@@ -97,42 +103,57 @@ exceeds 64 characters")
 
     def remplace_guillemets_motdepasse(self, ligne):
         """On remplace les mots de passe par un jeton"""
-        temoin = True
-        vraiefin = False
+        est_temoin = True
+        est_vraiefin = False
         motdepasse = '"'
-        while temoin:
+        while est_temoin:
             debut = ligne.find(' "')
             if debut != -1:
                 intermediaire = ligne[debut+2:]
-                #while vraiefin == False:
-                while not vraiefin: 
+                while not est_vraiefin: 
                     if '"' in intermediaire:
                         prochain = intermediaire.find('"')
                         motdepasse = motdepasse + intermediaire[:prochain+1]
                         if motdepasse[-2] == '\\':
                             intermediaire = intermediaire[prochain+1:]
                         else:
-                            vraiefin = True
+                            est_vraiefin = True
                     else:
-                        vraiefin = True
+                        est_vraiefin = True
                 chaine = motdepasse
                 motdepasse = '"'
-                vraiefin = False
+                est_vraiefin = False
                 nomjeton = 'jeton%s' % str(self._indice)
                 self._jetons[nomjeton] = chaine.strip('"')
                 ligne = ligne.replace(chaine, nomjeton, 1)
                 self._indice = self._indice + 1
             else:
-                temoin = False
+                est_temoin = False
         return ligne
 
-
     def fin_liste_ordres(self, num):
         """On traite la fin d'une liste d'ordres"""
         if (self._ordres[num] == linesep and 
-			self._script[-1] != self._entetel2) or (num + 1 == len(self._ordres) and 
+			self._script[-1] != self._entetel2) or (
+                    num + 1 == len(self._ordres) and 
 				self._script[-1] != self._entetel2):
-            if not self._commande:
+            # remplace le dernier numero de port du tunnel
+            if self._a_tunnel:
+                if self._pdest:
+                    self._script[self._dernierssh] = self._script[
+                        self._dernierssh].replace(
+                            '-L9999:127.0.0.1:9999','-L%s:127.0.0.1:%s' % (
+                                self._pdest, self._port),1)
+                else:
+                    if self._port:
+                        self._script[self._dernierssh] = self._script[
+                            self._dernierssh].replace(
+                                '127.0.0.1:9999','127.0.0.1:%s'% self._port, 1)
+                    else:
+                        self._script[self._dernierssh] = self._script[
+                            self._dernierssh].replace(
+                                '127.0.0.1:9999','127.0.0.1:22', 1)
+            if not self._a_commande:
                 self._script.append('interact +++ return%s' % linesep)
             else:
                 self._script.append('expect eof%s' % linesep)
@@ -145,40 +166,70 @@ exceeds 64 characters")
                 print message
                 sys.exit(1)
             self._script = [self._entetel1, self._entetel2]
-            self._premiereligne = True
-            self._commande = False
+            self._est_premiereligne = True
+            self._a_commande = False
 
     def interprete_ordres(self, num):
         """On interprete un ordre"""
-        port = ''
+        psource = ''
+        pdestination = ''
         ligne = self._ordres[num]
         if ligne != linesep:
-            if self._commande:
+            if self._a_commande:
                 self.traite_commande(ligne)
+            elif self._a_optstunnel: 
+                # on prepare les options du tunnel
+                optstunnel = OptionsTunnel(ligne)
+                psource, pdestination = \
+                    optstunnel.retourne_options()
+                nligne = '-L%s:127.0.0.1:%s' % (psource,
+                    pdestination)
+                self._pdest = pdestination
+                self._script[self._dernierssh] = self._script[
+                    self._dernierssh].replace(
+                        '-L9999:127.0.0.1:9999', nligne)
+                self._a_optstunnel = False
             else:
+                if ligne.split()[-1] == '-t':
+                    self._a_tunnel = True
+                if ligne.split()[-1] == '-ot':
+                    self._a_tunnel = True
+                    self._a_optstunnel = True
                 machine = ligne.split()[0]
                 self._machinefinale = ligne.split()[0].split('@')[-1]
                 if ':' in self._machinefinale:
-                    self._machinefinale, port = self._machinefinale.split(':')
+                    self._machinefinale, self._port = \
+self._machinefinale.split(':')
                     machine = ligne.split()[0].split(':')[0]
-                if self._premiereligne:
-                    if not port:
-                        self._script.append('spawn ssh %s %s%s' % (SSHOPTS, machine, linesep))
-                        self._premiereligne = False
-                    else:
-                        self._script.append('spawn ssh -p %s %s %s%s' %
-                                                 (port, SSHOPTS, machine, linesep))
-                        self._premiereligne = False
+                if self._a_tunnel:
+                    tunnelopts = '-L9999:127.0.0.1:9999'
                 else:
-                    if not port:
-                        self._script.append('send -- "ssh %s %s\\r"%s' % 
-                                            (SSHOPTS, machine, linesep))
+                    tunnelopts = ''
+
+                if self._est_premiereligne:
+                    if not self._port:
+                        self._script.append('spawn ssh %s %s %s%s' % (
+                            SSHOPTS, tunnelopts, machine, linesep))
                     else:
-                        self._script.append('send -- "ssh -p %s %s %s\\r"%s' %
-                                                     (port, SSHOPTS, machine, linesep))
+                        self._script.append('spawn ssh -p %s %s %s %s%s' %
+                            (self._port, SSHOPTS, tunnelopts, machine, linesep))
+                    self._est_premiereligne = False
+                else:
+                    if not self._port:
+                        self._script.append('send -- "ssh %s %s %s\\r"%s' % 
+                            (SSHOPTS, tunnelopts, machine, linesep))
+                    else:
+                        self._script.append(
+                            'send -- "ssh -p %s %s %s %s\\r"%s' %
+                            (self._port, SSHOPTS, tunnelopts, machine, 
+                                linesep))
+                if self._a_tunnel:
+                    self._dernierssh = len(self._script) - 1
                 if ligne.split()[-1] == '-c':
                     self.avec_commande(ligne)
-                    self._commande = True,
+                    self._a_commande = True
+                elif ligne.split()[-1] == '-t' or ligne.split()[-1] == '-ot':
+                    self.avec_commande(ligne)
                 else:
                     self.sans_commande(ligne)
 
@@ -210,7 +261,8 @@ exceeds 64 characters")
 
     def traite_commande(self, ligne):
         """Traite la commande contenue dans la ligne courante"""
-        self._script.append('send -- "%s\\r"%s' % (ligne.rstrip(linesep), linesep))
+        self._script.append('send -- "%s\\r"%s' 
+            % (ligne.rstrip(linesep), linesep))
         self.prepare_prompt()
 
     def change_utilisateur(self, utilisateur, motdepasse):
